import collections

import cocos
import pyglet
from pyglet.window import key

from cbl import rect

# some physics constants
JUMP = 250
SIDE_ACCEL = 500
MAX_SPEED = 100
MAX_FALL = 200
GRAVITY = -500


class HazImages(object):
    def load_images(self):
        for state in list(self.images):
            if not isinstance(self.images[state], basestring):
                continue
            image = pyglet.resource.image(self.images[state])
            self.images[state] = image
            if state.endswith('_left'):
                image = image.get_transform(flip_x=True)
                self.images[state.replace('_left', '_right')] = image
            elif state.endswith('_right'):
                image = image.get_transform(flip_x=True)
                self.images[state.replace('_right', '_left')] = image


class Actor(HazImages):
    state = 'standing'
    direction = 1

    def __init__(self, position, **other):
        self.__dict__.update(other)

        self.load_images()

        image = self.images[self.state]
        self.sprite = cocos.sprite.Sprite(image, 
            position=position, anchor=(image.width//2, 0))
        self.old_rect = self.sprite.get_rect()

        self.dx, self.dy = 0, 0
        self.ddx, self.ddy = 0, 0
        self.last_move = 0
        self.target_position = position

    def change_state(self, state):
        if state != self.state:
            # XXX call state_<state>...?
            self.state = state
            if state in self.images:
                self.sprite.image = self.images[state]
            else:
                if self.direction < 0:
                    i = self.sprite.image = self.images[state+'_left']
                    self.sprite.image_anchor_x = i.width - 8
                else:
                    self.sprite.image = self.images[state+'_right']
                    self.sprite.image_anchor_x = 8

    def update(self, dt, scene):
        pass

    def move_to(self, position):
        self.target_position = position


class Partner(Actor):
    is_dead = False
    images = dict(
        standing='partner-standing.png',
        move_right='partner-standing.png',
        dead='partner-dead.png',
    )

    def died(self):
        self.is_dead = True
        x, y = self.sprite.position
        image = self.images['dead']
        self.sprite.image = image
        self.sprite.anchor = (image.width//2, 0)
        self.sprite.position = x, y


class ShooterNPC(Actor):
    images = dict(
        standing='baddie-standing.png',
        move_right='baddie-standing.png',
        shooting_right='baddie-shooting.png',
    )


class GuardNPC(Actor):
    images = dict(
        standing='baddie-torch.png',
        move_right='baddie-torch.png',
        shooting_right='baddie-shooting.png',
    )
    SPEED = 50

    delay = 0
    def update(self, dt, scene):
        self.old_rect = self.sprite.get_rect()
        if self.state == 'standing':
            self.change_state('move')
        if self.delay > 0:
            self.delay -= dt
            return

        p = scene.actors['player']
    
        # move from self.point_a to self.point_b and back again
        # if the player's in range in the correct direction then shoot
        pb = p.sprite.get_rect()
        if self.state == 'move':
            self.sprite.x += dt * self.SPEED * self.direction
            if self.direction > 0:
                hb = rect.Rect(self.sprite.x, self.sprite.y, 90, 32)
                if p.alive and not p.hidden and hb.intersects(pb):
                    self.change_state('shooting')
                elif self.sprite.x > self.point_b[0]:
                    self.direction *= -1
                    s, self.state = self.state, None
                    self.change_state(s)
            else:
                hb = rect.Rect(self.sprite.x-90, self.sprite.y, 90, 32)
                if p.alive and not p.hidden and hb.intersects(pb):
                    self.change_state('shooting')
                elif self.sprite.x < self.point_a[0]:
                    self.direction *= -1
                    s, self.state = self.state, None
                    self.change_state(s)
        elif p.alive and not p.hidden:
            scene.shoot(self)
            self.delay = .5
        else:
            self.change_state('move')


class Player(HazImages, cocos.layer.Layer):
    is_event_handler = True
    has_keys = False
    alive = True
    images = dict(
        standing='player-standing.png',
        move_right='player-standing.png',
        shooting_left='player-shooting.png',
    )

    def __init__(self):
        super(Player, self).__init__()
        self.dx, self.dy = 0, 0
        self.ddx, self.ddy = 0, 0
        self.last_move = 0
        self.has_gun = False

        self.load_images()

    def get_scene(self):
        return self.parent.parent.parent

    def on_enter(self):
        super(Player, self).on_enter()
        self.keys = collections.defaultdict(int)
        r = self.get_scene().get_box('target_player_start')
        image = self.images['standing']
        self.sprite = cocos.sprite.Sprite(image, 
            position=r.midbottom, anchor=(image.width//2, 0))
        self.add(self.sprite)
        self.old_rect = self.sprite.get_rect()

    def on_key_press(self, symbol, modifiers):
        self.keys[symbol] = True
        if symbol == key.SPACE and self.shooting:
            self.get_scene().player_shoot()

    def on_key_release(self, symbol, modifiers):
        if symbol == key.E:
            s = self.get_scene()
            if s.action_bubble is not None:
                s.action_bubble.action()
        self.keys[symbol] = False

    allow_movement = True
    hidden = False
    def do_hide(self):
        self.hidden = True
        self.sprite.opacity = 0
        self.allow_movement = False
    def do_unhide(self):
        self.hidden = False
        self.sprite.opacity = 255
        self.allow_movement = True

    shooting = False
    def update(self, dt, scene):
        self.old_rect = self.sprite.get_rect()
        move = self.keys[key.RIGHT] - self.keys[key.LEFT]

        on_ladder = scene.on_ladder(self.sprite.get_rect())
        updown = 0
        if on_ladder:
            updown = self.keys[key.UP] - self.keys[key.DOWN]

        if self.keys[key.LSHIFT] and not self.shooting:
            image = self.images['shooting']
            self.sprite.image = image
            self.sprite.anchor = (image.width//2, 0)
            self.shooting = True
        elif not self.keys[key.LSHIFT] and self.shooting:
            image = self.images['standing']
            self.sprite.image = image
            self.sprite.anchor = (image.width//2, 0)
            self.shooting = False

        if not self.allow_movement:
            move = 0

        if move > 0:
            if self.dx < 0: self.dx = 0                     # TURN
            self.ddx = SIDE_ACCEL
            self.last_move = move
        elif move < 0:
            if self.dx > 0: self.dx = 0                     # TURN
            self.ddx = -SIDE_ACCEL
            self.last_move = move
        else:
            self.ddx = self.dx = 0                          # STOP

        self.dx += dt * self.ddx
        if self.dx > MAX_SPEED: self.dx = MAX_SPEED           # max run speed
        elif self.dx < -MAX_SPEED: self.dx = -MAX_SPEED
        # XXX remember actual desired float position and calculate on that
        self.sprite.x = float(self.sprite.x + dt * self.dx)

        # vertical movement
        if on_ladder:
            self.dy = 100 * updown
        else:
            self.dy += dt * GRAVITY
        if self.dy < -MAX_FALL: self.dy = -MAX_FALL         # terminal velocity
        self.sprite.y = float(self.sprite.y + dt * self.dy)

        scene.level_collide(dt, self, skip_ledge=on_ladder)
        scene.level_trigger(dt, self)

