import os
import random

import pyglet
import cocos
from cocos.director import director
from cocos.actions import Delay, CallFunc, FadeOut

from cbl import svg_layer
from cbl import speech
from cbl import layer_message

class SceneLayer(cocos.layer.Layer):
    focus_player = False

    def __init__(self):
        super(SceneLayer, self).__init__()

        # load the information from the map SVG
        path = os.path.join(pyglet.resource.location(self.map_file).path,
            self.map_file)
        self.scroller = cocos.layer.ScrollingManager()
        self.add(self.scroller)
        
        self.map_layers = svg_layer.load(path)
        self.scroller.add(self.map_layers['background'], z=0)
        self.action_layer = cocos.layer.ScrollableLayer()
        self.scroller.add(self.action_layer, z=1)
        self.scroller.add(self.map_layers['foreground'], z=2)
        self.gui_layer = cocos.layer.ScrollableLayer()
        self.scroller.add(self.gui_layer, z=3)
#        self.scroller.add(self.map_layers['triggers'], z=4)
        self.triggers = self.map_layers['triggers'].svg
        if 'focus' in self.triggers.paths_by_label:
            t = self.triggers.paths_by_label['focus'][0].get_box()
            self.scroller.set_focus(*t.center)
        self.blockers = [path.get_box()
            for path in self.triggers.paths_by_label['blocker']]
        self.ledges = [path.get_box()
            for path in self.triggers.paths_by_label['ledge']]
        self.game_bounds = [path.get_box()
            for path in self.triggers.paths_by_label['game_bounds']][0]

        self.actors = {}
        self.current_triggers = {}
        self.triggers_disabled = {}

        self.open_scene()
        self.schedule(self.update)

    def remove_blocker(self, blocker):
        self.triggers_disabled[blocker] = True
        self.blockers = [path.get_box()
            for path in self.triggers.paths_by_label['blocker']
            if path.id not in self.triggers_disabled]

    def add_actor(self, name, klass, location, **kw):
        r = self.get_box(location)
        a = self.actors[name] = klass(r.midbottom, **kw)
        if hasattr(a, 'sprite'):
            self.action_layer.add(a.sprite)
        return a

    def on_exit(self):
        self.close_scene()
        self.unschedule(self.update)
        super(SceneLayer, self).on_exit()

    def you_died(self):
        self.actors['player'].alive = False
        self.actors['player'].allow_movement = False
        self.add(layer_message.Message('You Died'), z=10)

    def you_failed(self):
        self.add(layer_message.Message('You Failed'), z=10)

    def open_scene(self):
        pass
    def close_scene(self):
        pass

    def get_box(self, label):
        return self.triggers.paths_by_label[label][0].get_box()

    def say(self, actor, text, callback=None, duration=2):
        s = self.actors[actor].sprite
        pos = (s.width//2, s.height+2)
        s.add(speech.SpeechLayer(text, pos, callback, duration))

    def hint(self, text, duration=5):
        p = self.triggers.paths_by_label['hint'][0].get_box().center
        t = cocos.text.Label(text, position=p, width=300, multiline=True,
            anchor_x='center', anchor_y='top', color=(255,255,255,255))
        self.gui_layer.add(t)
        self.do(Delay(duration) +
            CallFunc(lambda g=self.gui_layer:g.remove(t)))

    def add_bullet(self, source, direction, target, hit):
        if direction > 0:
            sx, sy = source.midright
        else:
            sx, sy = source.midleft

        tx, ty = target.center
        if not hit:
            dx = dy = 0
            while dx + dy < 10:
                dx = random.randint(-100, 100)
                dy = random.randint(-10, 50)
            tx += dx
            ty += dy

        # XXX limit tx, ty to the gameplay area

        # XXX draw a bullet flying

        s = cocos.sprite.Sprite('bullet_spark.png', (tx, ty))
        self.gui_layer.add(s)
        s.do(FadeOut(duration=0.1))

    action_bubble = None
    def enable_action(self, text, action):
        position = 0, -8
        if self.action_bubble is not None:
            self.action_bubble.parent.remove(self.action_bubble)
        self.action_bubble = speech.ActionLayer(text, position)
        self.action_bubble.action = action
        self.actors['player'].sprite.add(self.action_bubble)
    def disable_action(self):
        if self.action_bubble is None:
            return
        self.action_bubble.parent.remove(self.action_bubble)
        self.action_bubble = None

    def update(self, dt):
        for actor in self.actors.values():
            actor.update(dt, self)
        if self.focus_player:
            self.scroller.set_focus(*self.actors['player'].sprite.get_rect().midbottom)

    def new_scene(self, scene):
        director.replace(cocos.scene.Scene(scene))

    def on_ladder(self, box):
        for path in self.triggers.paths_by_label.get('ladder', []):
            if path.get_box().intersects(box):
                return True
        return False

    def level_collide(self, dt, actor, skip_ledge=False):
        # WORLD GEOMETRY (GROUND) COLLISIONS
        r = actor.sprite.get_rect()
        olr = actor.old_rect
        for box in self.blockers:
            if not r.intersects(box):
                continue
            if olr.bottom >= box.top and r.bottom < box.top:
                r.bottom = box.top
                self.dy = 0
            if olr.right <= box.left and r.right > box.left:
                self.dx = 0
                r.right = box.left
            if olr.left >= box.right and r.left < box.right:
                self.dx = 0
                r.left = box.right
            if olr.top <= box.bottom and r.top > box.bottom:
                self.dy = 0
                r.top = box.bottom

        # LEDGE COLLISIONS
        if not skip_ledge:
            for box in self.ledges:
                if not r.intersects(box):
                    continue
                if olr.bottom >= box.top and r.bottom < box.top:
                    r.bottom = box.top
                    self.dy = 0

        # game boundary
        self.game_bounds.constrain(r)

        actor.sprite.position = r.midbottom

    def level_trigger(self, dt, actor):
        # trigger events
        r = actor.sprite.get_rect()
        action = None
        for shape in self.triggers.paths:
            if not shape.label.startswith('trigger_'):
                continue
            if shape.label in self.triggers_disabled:
                continue
            if r.intersects(shape.get_box()):
                if shape.label not in self.current_triggers:
                    action = getattr(self, shape.label)
                    print 'TRIGGER', 
                    self.current_triggers[shape.label] = True
            elif shape.label in self.current_triggers:
                del self.current_triggers[shape.label]
                if hasattr(self, 'un' + shape.label):
                    action = getattr(self, 'un'+shape.label)
                    print 'UNTRIGGER', 
            if action is not None:
                print action
                action()
                break

